%Library containing derivatives <plibrary.mat> should be loaded. 
%load plibrary.mat;

%Inputs should be stored as variables in the current workspace: 
%Nem: Number of emitters to attempt a fit
%px: Size of a pixel in nm
%isize: Number of pixels in input image is (2*isize+1)^2
%imgin: Input image, a (2*isize+1)x(2*isize+1) array
%varbkgd: Gain corrected variance in input image, see Eq. 14. Can be scalar
%   or array of same size as input image. 
%p: Initial parameter guess, a (5*Nem+1)x1 column vector: [N x0 y0 alpha
%beta b]', where first 5 parameters are repeated Nem times

%All variables generated are stored in current workspace. 
%Variables of interest below can be treated as outputs: 
%p: Optimal parameters in MLE result, same format as input
%mu: Best fit image given p parameters, same format as imgin
%muvarbkgd: mu + varbkgd
%cont2: Flag determining whether Newton's method converged. See code for
%   keys. 
%g: Derivative of mu with respect to parameters, (2*isize+1)^2x(5*Nem+1)
%   array. See Eq. 21, g(i,j) = dmu_i/dtheta_j. 
%H: Hessian of log-likelihood, (5*Nem+1)x(5*Nem+1) array. See Eq. 21,
%   H(j,k)


[xin yin] = meshgrid((-(isize+.5):.5:(isize+.5))*px,(-(isize+.5):.5:(isize+.5))*px); %Coordinates (in nm) of all pixels (and half pixels) in image, where center pixel is (x,y) = (0,0), y increases going down, x increases going right
cindex = 2:2:(length(xin)-1); % xin(cindex,cindex) gives x-coordinates of all pixels in image, discards the half pixels. 

wspan2 = (2*isize+1)^2; %Convenient constant
maxA = max(imgin(:))*2*pi*(115/px)^2; %Rough estimate for intensity of brightest pixel in input
noise = 6; %Rough estimate for standard deviation of background intensity. Adjust if necessary

%This loop uses Newton's method to maximize the Poisson
%likelihood by adjusting p. 
cont2 = 1; %Flag determining whether Newton's method has converged. 
stepcount = 0; %Counts the number of steps in Newton's method. 
while cont2 > 0
    stepcount = stepcount + 1;

    %This block calculates the gradient of the log-likelihood and
    %the Hessian, with respect to the parameters in p. 
    %Gradient is dL, a 5*Nem+1 length column vector. 
    %Hessian is H, a 5*Nem+1 square matrix. 
    mu = zeros(2*isize+1,2*isize+1);
    g = zeros(wspan2,5*Nem+1);
    H = zeros(wspan2,5*Nem+1,5*Nem+1);
    maxpix = zeros(Nem,1);
    for k = 1:Nem %Loops through the Nem emitters
        %p0, g0, H0 are parameters for given emitter, subarrays of p, g,
        %and H for convenience. 
        p0 = p((k-1)*5+(1:5));
        g0 = zeros(wspan2,5);
        H0 = zeros(wspan2,5,5);
        alpha = p0(4);
        beta = p0(5);
        
        %Determine polar coordinates of each pixel. 
        rho = sqrt((xin-p0(2)).^2+(yin-p0(3)).^2);
        phi = atan2(yin-p0(3),xin-p0(2));
        rhosq = rho.^2;
        rhocu = rho.^3;
        cosphi = cos(phi);
        sinphi = sin(phi);
        cossqphi = cos(phi).^2;
        sinsqphi = sin(phi).^2;
        cos2phi = cos(2*phi);
        sin2phi = sin(2*phi);
        cos3phi = cos(3*phi);
        sin3phi = sin(3*phi);
        
        %Pre-store some values for convenience. 
        cosphialpha = cos(phi-alpha);
        cos2phi2alpha = cos(2*phi-2*alpha);
        sinphialpha = sin(phi-alpha);
        sin2phi2alpha = sin(2*phi-2*alpha);
        cosbeta = cos(beta);
        sinbeta = sin(beta);
        cos2beta = cos(2*beta);
        sin2beta = sin(2*beta);
        cos4beta = cos(4*beta);
        
        %Denominator of Eq. 1. 
        Norm = 1/(Ipara*sinbeta^2+Iperp*cosbeta^2);
        
        %Determine values of p functions (Eq. 2) by interpolating from
        %library. 
        rhors = reshape(rho,(4*isize+3)^2,1); %Reshape rho into column vector
        drhop = rhop(2)-rhop(1);
        rhoindex = floor((rhors-rhop(1))/drhop) + 1;
        rhofrac = (rhors-rhop(1))/drhop - rhoindex + 1;
        %Interpolate and reshape p functions and their derivatives. 
        ippara = reshape(ppara(rhoindex).*(1-rhofrac) + ppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        ipperp = reshape(pperp(rhoindex).*(1-rhofrac) + pperp(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        ipcross = reshape(pcross(rhoindex).*(1-rhofrac) + pcross(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        idelppara = reshape(delppara(rhoindex).*(1-rhofrac) + delppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iDppara = reshape(Dppara(rhoindex).*(1-rhofrac) + Dppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iDpperp = reshape(Dpperp(rhoindex).*(1-rhofrac) + Dpperp(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iDpcross = reshape(Dpcross(rhoindex).*(1-rhofrac) + Dpcross(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iDdelppara = reshape(Ddelppara(rhoindex).*(1-rhofrac) + Ddelppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD2ppara = reshape(D2ppara(rhoindex).*(1-rhofrac) + D2ppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD2pperp = reshape(D2pperp(rhoindex).*(1-rhofrac) + D2pperp(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD2pcross = reshape(D2pcross(rhoindex).*(1-rhofrac) + D2pcross(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD2delppara = reshape(D2delppara(rhoindex).*(1-rhofrac) + D2delppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD3ppara = reshape(D3ppara(rhoindex).*(1-rhofrac) + D3ppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD3pperp = reshape(D3pperp(rhoindex).*(1-rhofrac) + D3pperp(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD3pcross = reshape(D3pcross(rhoindex).*(1-rhofrac) + D3pcross(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);
        iD3delppara = reshape(D3delppara(rhoindex).*(1-rhofrac) + D3delppara(rhoindex+1).*rhofrac,4*isize+3,4*isize+3);

        f = Norm*(sinbeta^2*(ippara+cos2phi2alpha.*idelppara) + sinbeta*cosbeta*cosphialpha.*ipcross + cosbeta^2*ipperp);
        
        %Names of all derivatives: D3fdrho2dphi means third derivative of
        %f, twice with rho, once with phi. 
        Dfdrho = Norm*(sinbeta^2*(iDppara+cos2phi2alpha.*iDdelppara) + sinbeta*cosbeta*cosphialpha.*iDpcross + cosbeta^2*iDpperp);
        Dfdphi = Norm*(-2*sinbeta^2*sin2phi2alpha.*idelppara - sinbeta*cosbeta*sinphialpha.*ipcross);
        D2fdrho2 = Norm*(sinbeta^2*(iD2ppara+cos2phi2alpha.*iD2delppara) + sinbeta*cosbeta*cosphialpha.*iD2pcross + cosbeta^2*iD2pperp);
        D2fdrhodphi = Norm*(-2*sinbeta^2*sin2phi2alpha.*iDdelppara - sinbeta*cosbeta*sinphialpha.*iDpcross);
        D2fdphi2 = Norm*((-4*sinbeta^2*cos2phi2alpha.*idelppara - sinbeta*cosbeta*cosphialpha.*ipcross));
        D3fdrho3 = Norm*(sinbeta^2*(iD3ppara+cos2phi2alpha.*iD3delppara) + sinbeta*cosbeta*cosphialpha.*iD3pcross + cosbeta^2*iD3pperp);
        D3fdrho2dphi = Norm*(-2*sinbeta^2*sin2phi2alpha.*iD2delppara - sinbeta*cosbeta*sinphialpha.*iD2pcross);
        D3fdrhodphi2 = Norm*((-4*sinbeta^2*cos2phi2alpha.*iDdelppara - sinbeta*cosbeta*cosphialpha.*iDpcross));
        D3fdphi3 = Norm*((8*sinbeta^2*sin2phi2alpha.*idelppara + sinbeta*cosbeta*sinphialpha.*ipcross));
        
        Dfdx = Dfdrho.*cosphi - Dfdphi.*sinphi./rho;
        Dfdy = Dfdrho.*sinphi + Dfdphi.*cosphi./rho;
        D2fdx2 = D2fdrho2.*cossqphi - D2fdrhodphi.*sin2phi./rho + D2fdphi2.*sinsqphi./rhosq + Dfdrho.*sinsqphi./rho + Dfdphi.*sin2phi./rhosq;
        D2fdy2 = D2fdrho2.*sinsqphi + D2fdrhodphi.*sin2phi./rho + D2fdphi2.*cossqphi./rhosq + Dfdrho.*cossqphi./rho - Dfdphi.*sin2phi./rhosq;
        D3fdx2dy = D3fdrho3.*sinphi.*cossqphi + D3fdrho2dphi.*(cosphi+3*cos3phi)./(4*rho) + D3fdrhodphi2.*(sinphi-3*sin3phi)./(4*rhosq) + D3fdphi3.*sinsqphi.*cosphi./rhocu...
            + D2fdrho2.*(sinphi-3*sin3phi)./(4*rho) + D2fdrhodphi.*(cosphi-9*cos3phi)./(4*rhosq) - D2fdphi2.*(sinphi-3*sin3phi)./(2*rhocu)...
            - Dfdrho.*(sinphi-3*sin3phi)./(4*rhosq) + 2*Dfdphi.*cos3phi./rhocu;
        D3fdxdy2 = D3fdrho3.*sinsqphi.*cosphi - D3fdrho2dphi.*(sinphi+3*sin3phi)./(4*rho) + D3fdrhodphi2.*(cosphi+3*cos3phi)./(4*rhosq) - D3fdphi3.*sinphi.*cossqphi./rhocu...
            + D2fdrho2.*(cosphi+3*cos3phi)./(4*rho) - D2fdrhodphi.*(sinphi+9*sin3phi)./(4*rhosq) - D2fdphi2.*(cosphi+3*cos3phi)./(2*rhocu)...
            - Dfdrho.*(cosphi+3*cos3phi)./(4*rhosq) + 2*Dfdphi.*sin3phi./rhocu;
        
        %See Eq. 12, this is mu before multiplying by N and adding b. 
        fadj = f(cindex,cindex)*px^2 + (D2fdx2(cindex,cindex)+D2fdy2(cindex,cindex))*px^4/24;
        
        %Add contribution of this emitter to mu. 
        mu = mu + p0(1)*fadj;
        
        %Determine derivatives of mu. 
        g0(:,1) = reshape(fadj,wspan2,1);
        
        Dfadjdx0 = (f(cindex,cindex-1)-f(cindex,cindex+1))*px + (D2fdy2(cindex,cindex-1)-D2fdy2(cindex,cindex+1))*px^3/24;
        Dfadjdy0 = (f(cindex-1,cindex)-f(cindex+1,cindex))*px + (D2fdx2(cindex-1,cindex)-D2fdx2(cindex+1,cindex))*px^3/24;

        g0(:,2) = reshape(p0(1)*Dfadjdx0,wspan2,1);
        g0(:,3) = reshape(p0(1)*Dfadjdy0,wspan2,1);
        
        D3fdphidx2 = D3fdrho2dphi.*cossqphi - D3fdrhodphi2.*sin2phi./rho + D3fdphi3.*sinsqphi./rhosq + D2fdrhodphi.*sinsqphi./rho + D2fdphi2.*sin2phi./rhosq;
        D3fdphidy2 = D3fdrho2dphi.*sinsqphi + D3fdrhodphi2.*sin2phi./rho + D3fdphi3.*cossqphi./rhosq + D2fdrhodphi.*cossqphi./rho - D2fdphi2.*sin2phi./rhosq;
        Dfadjdphi = Dfdphi(cindex,cindex)*px^2 + (D3fdphidx2(cindex,cindex)+D3fdphidy2(cindex,cindex))*px^4/24;
        
        g0(:,4) = reshape(-p0(1)*Dfadjdphi,wspan2,1);
        
        Dfdbeta = Norm^2*(sin2beta*Iperp*(ippara+cos2phi2alpha.*idelppara) - sin2beta*Ipara*ipperp + .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*cosphialpha.*ipcross);
        D2fdrhodbeta = Norm^2*(sin2beta*Iperp*(iDppara+cos2phi2alpha.*iDdelppara) - sin2beta*Ipara*iDpperp + .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*cosphialpha.*iDpcross);
        D2fdphidbeta = Norm^2*(-2*sin2beta*Iperp*sin2phi2alpha.*idelppara - .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*sinphialpha.*ipcross);
        D3fdrho2dbeta = Norm^2*(sin2beta*Iperp*(iD2ppara+cos2phi2alpha.*iD2delppara) - sin2beta*Ipara*iD2pperp + .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*cosphialpha.*iD2pcross);
        D3fdrhodphidbeta = Norm^2*(-2*sin2beta*Iperp*sin2phi2alpha.*iDdelppara - .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*sinphialpha.*iDpcross);
        D3fdphi2dbeta = Norm^2*(-4*sin2beta*Iperp*cos2phi2alpha.*idelppara - .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*cosphialpha.*ipcross);
        
        D3fdbetadx2 = D3fdrho2dbeta.*cossqphi - D3fdrhodphidbeta.*sin2phi./rho + D3fdphi2dbeta.*sinsqphi./rhosq + D2fdrhodbeta.*sinsqphi./rho + D2fdphidbeta.*sin2phi./rhosq;
        D3fdbetady2 = D3fdrho2dbeta.*sinsqphi + D3fdrhodphidbeta.*sin2phi./rho + D3fdphi2dbeta.*cossqphi./rhosq + D2fdrhodbeta.*cossqphi./rho - D2fdphidbeta.*sin2phi./rhosq;
        Dfadjdbeta = Dfdbeta(cindex,cindex)*px^2 + (D3fdbetadx2(cindex,cindex)+D3fdbetady2(cindex,cindex))*px^4/24;
        
        g0(:,5) = reshape(p0(1)*Dfadjdbeta,wspan2,1);
        
        %Determine Hessian using second derivatives of mu. 
        H0(:,2:5,1) = g0(:,2:5)/p0(1);
        
        D2fadjdx02 = (Dfdx(cindex,cindex+1)-Dfdx(cindex,cindex-1))*px + (D3fdxdy2(cindex,cindex+1)-D3fdxdy2(cindex,cindex-1))*px^3/24;
        D2fadjdx0dy0 = f(cindex+1,cindex+1) - f(cindex-1,cindex+1) - f(cindex+1,cindex-1) + f(cindex-1,cindex-1);
        D2fadjdy02 = (Dfdy(cindex+1,cindex)-Dfdy(cindex-1,cindex))*px + (D3fdx2dy(cindex+1,cindex)-D3fdx2dy(cindex-1,cindex))*px^3/24;
        
        H0(:,2,2) = reshape(p0(1)*D2fadjdx02,wspan2,1);
        H0(:,3,2) = reshape(p0(1)*D2fadjdx0dy0,wspan2,1);
        H0(:,3,3) = reshape(p0(1)*D2fadjdy02,wspan2,1);
        
        D4fdrho2dphi2 = Norm*((-4*sinbeta^2*cos2phi2alpha.*iD2delppara - sinbeta*cosbeta*cosphialpha.*iD2pcross));
        D4fdrhodphi3 = Norm*((8*sinbeta^2*sin2phi2alpha.*iDdelppara + sinbeta*cosbeta*sinphialpha.*iDpcross));
        D4fdphi4 = Norm*((16*sinbeta^2*cos2phi2alpha.*idelppara + sinbeta*cosbeta*cosphialpha.*ipcross));
        
        D4fdphi2dx2 = D4fdrho2dphi2.*cossqphi - D4fdrhodphi3.*sin2phi./rho + D4fdphi4.*sinsqphi./rhosq + D3fdrhodphi2.*sinsqphi./rho + D3fdphi3.*sin2phi./rhosq;
        D4fdphi2dy2 = D4fdrho2dphi2.*sinsqphi + D4fdrhodphi3.*sin2phi./rho + D4fdphi4.*cossqphi./rhosq + D3fdrhodphi2.*cossqphi./rho - D3fdphi3.*sin2phi./rhosq;
        
        D2fadjdphidx0 = (Dfdphi(cindex,cindex-1)-Dfdphi(cindex,cindex+1))*px + (D3fdphidy2(cindex,cindex-1)-D3fdphidy2(cindex,cindex+1))*px^3/24;
        D2fadjdphidy0 = (Dfdphi(cindex-1,cindex)-Dfdphi(cindex+1,cindex))*px + (D3fdphidx2(cindex-1,cindex)-D3fdphidx2(cindex+1,cindex))*px^3/24;
        D2fadjdphi2 = D2fdphi2(cindex,cindex)*px^2 + (D4fdphi2dx2(cindex,cindex)+D4fdphi2dy2(cindex,cindex))*px^4/24;
        
        H0(:,4,2) = reshape(-p0(1)*D2fadjdphidx0,wspan2,1);
        H0(:,4,3) = reshape(-p0(1)*D2fadjdphidy0,wspan2,1);
        H0(:,4,4) = reshape(p0(1)*D2fadjdphi2,wspan2,1);
        
        D4fdrho2dphidbeta = Norm^2*(-2*sin2beta*Iperp*sin2phi2alpha.*iD2delppara - .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*sinphialpha.*iD2pcross);
        D4fdrhodphi2dbeta = Norm^2*(-4*sin2beta*Iperp*cos2phi2alpha.*iDdelppara - .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*cosphialpha.*iDpcross);
        D4fdphi3dbeta = Norm^2*(8*sin2beta*Iperp*sin2phi2alpha.*idelppara + .5*(Iperp-Ipara+(Ipara+Iperp)*cos2beta)*sinphialpha.*ipcross);
        
        D4fdphidbetadx2 = D4fdrho2dphidbeta.*cossqphi - D4fdrhodphi2dbeta.*sin2phi./rho + D4fdphi3dbeta.*sinsqphi./rhosq + D3fdrhodphidbeta.*sinsqphi./rho + D3fdphi2dbeta.*sin2phi./rhosq;
        D4fdphidbetady2 = D4fdrho2dphidbeta.*sinsqphi + D4fdrhodphi2dbeta.*sin2phi./rho + D4fdphi3dbeta.*cossqphi./rhosq + D3fdrhodphidbeta.*cossqphi./rho - D3fdphi2dbeta.*sin2phi./rhosq;
        
        D2fdbeta2 = Norm^3*(((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*((ippara+cos2phi2alpha.*idelppara)*Iperp-ipperp*Ipara)...
            + .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*cosphialpha.*ipcross);
        D3fdrhodbeta2 = Norm^3*(((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*((iDppara+cos2phi2alpha.*iDdelppara)*Iperp-iDpperp*Ipara)...
            + .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*cosphialpha.*iDpcross);
        D3fdphidbeta2 = Norm^3*(-2*((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*Iperp*sin2phi2alpha.*idelppara...
            - .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*sinphialpha.*ipcross);
        D4fdrho2dbeta2 = Norm^3*(((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*((iD2ppara+cos2phi2alpha.*iD2delppara)*Iperp-iD2pperp*Ipara)...
            + .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*cosphialpha.*iD2pcross);
        D4fdrhodphidbeta2 = Norm^3*(-2*((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*Iperp*sin2phi2alpha.*iDdelppara...
            - .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*sinphialpha.*iDpcross);
        D4fdphi2dbeta2 = Norm^3*(-4*((Ipara+Iperp)*cos2beta+.5*(Ipara-Iperp)*(cos4beta-3))*Iperp*cos2phi2alpha.*idelppara...
            - .5*sin2beta*((Ipara-Iperp)^2-4*Ipara*Iperp+(Iperp^2-Ipara^2)*cos2beta)*cosphialpha.*ipcross);
        
        D4fdbeta2dx2 = D4fdrho2dbeta2.*cossqphi - D4fdrhodphidbeta2.*sin2phi./rho + D4fdphi2dbeta2.*sinsqphi./rhosq + D3fdrhodbeta2.*sinsqphi./rho + D3fdphidbeta2.*sin2phi./rhosq;
        D4fdbeta2dy2 = D4fdrho2dbeta2.*sinsqphi + D4fdrhodphidbeta2.*sin2phi./rho + D4fdphi2dbeta2.*cossqphi./rhosq + D3fdrhodbeta2.*cossqphi./rho - D3fdphidbeta2.*sin2phi./rhosq;
        
        D2fadjdbetadx0 = (Dfdbeta(cindex,cindex-1)-Dfdbeta(cindex,cindex+1))*px + (D3fdbetady2(cindex,cindex-1)-D3fdbetady2(cindex,cindex+1))*px^3/24;
        D2fadjdbetady0 = (Dfdbeta(cindex-1,cindex)-Dfdbeta(cindex+1,cindex))*px + (D3fdbetadx2(cindex-1,cindex)-D3fdbetadx2(cindex+1,cindex))*px^3/24;
        D2fadjdphidbeta = D2fdphidbeta(cindex,cindex)*px^2 + (D4fdphidbetadx2(cindex,cindex)+D4fdphidbetady2(cindex,cindex))*px^4/24;
        D2fadjdbeta2 = D2fdbeta2(cindex,cindex)*px^2 + (D4fdbeta2dx2(cindex,cindex)+D4fdbeta2dy2(cindex,cindex))*px^4/24;
        
        H0(:,5,2) = reshape(p0(1)*D2fadjdbetadx0,wspan2,1);
        H0(:,5,3) = reshape(p0(1)*D2fadjdbetady0,wspan2,1);
        H0(:,5,4) = reshape(-p0(1)*D2fadjdphidbeta,wspan2,1);
        H0(:,5,5) = reshape(p0(1)*D2fadjdbeta2,wspan2,1);
        
        %Fill larger g and H with data from this emitter. 
        g(:,(k-1)*5+(1:5)) = g0;
        H(:,(k-1)*5+(1:5),(k-1)*5+(1:5)) = H0;
        
        maxpix(k) = max(max(fadj))*p0(1);
    end
    mu = mu + p(5*Nem+1);
    muvarbkgd = mu + varbkgd;
    imginvarbkgd = imgin + varbkgd;
    temp = imginvarbkgd./muvarbkgd-1; %See Eq. 21
    temp2 = imginvarbkgd./muvarbkgd.^2;
    temp = temp(:);
    temp2 = temp2(:);
    g(:,5*Nem+1) = 1;
    H = squeeze(sum(bsxfun(@times,H,temp),1));
    H = H + tril(H,-1)';
    H = H - squeeze(sum(bsxfun(@times,bsxfun(@times,g,permute(g,[1 3 2])),temp2),1)); %H is symmetric
    dL = sum(bsxfun(@times,g,temp))';

    %Calculates the Newton's method step, which is -inv(H)*dL.
    %First checks if H is negative definite. If not, subtracts
    %an identity matrix from H to make it so. 
    maxeig = max(eig(H));
    if maxeig < -1e-10
        dp = -H\dL;
    else
        dp = -(H+(-maxeig-1e-10)*eye(5*Nem+1))\dL;
    end

    %Adjusts dp such that the parameters don't vary too quickly
    %in each step. 
    if p(end)+dp(end) < 0 %Makes sure background parameter does not become negative. 
        dp = abs(p(end)/dp(end))*dp*.9;
    end
    if Nem > 0
        dx = max(abs([dp(2:5:(5*Nem));dp(3:5:(5*Nem))]));
        if dx > px %Makes sure x and y positions do not move more than 1 pixel. 
            dp = dp/dx*px;
        end
        dA = max(abs(dp(1:5:(5*Nem))));
        if dA > .1*maxA %Makes sure intensity parameters do not adjust more than 0.1*maxA, where maxA is the highest intensity pixel in the sub-ROI. 
            dp = dp/dA*(.1*maxA);
        end
        [dbeta ind] = max(abs(dp(5:5:(5*Nem))));
        ind = ind*5;
        if dbeta > abs(.75*p(ind)) %Makes sure beta parameters do not vary too quickly. 
            dp = dp/dbeta*abs(.75*p(ind));
        end
    end

    p = p + dp; %Iterate according to Newton's method. 
    
    %Determine whether fit converged. cont2 is 0 if yes,
    %1 if no, negative if some error occurred and fit should be
    %rejected, which is indicated by flags below. 
    cont2 = max(abs(dp./p)) > .0001 && max(abs(dp)) > .001;
    if Nem > 0
        if max(abs([p(2:5:(5*Nem));p(3:5:(5*Nem))])) > isize*px+300 %Reject if one of the dyes is way outside the sub-ROI. 
            cont2 = -1;
        end
        if min(maxpix) < .5*noise %Reject if one of the dye intensities is way below noise. 
            cont2 = -2;
        end
    end
    if stepcount > 150 %Reject if maximization is taking too many steps. 
        cont2 = -3;
    end
end
if maxeig > 1e-10
    cont2 = -5; %Reject if Hessian is not negative semi-definite at best fit. 
end

%Handles a few degenerate cases of the orientation parameters. 
arange = 4:5:(5*Nem);
brange = 5:5:(5*Nem);
p(brange) = p(brange) - pi*floor(p(brange)/pi);
corr = p(brange) > pi/2;
p(brange) = corr.*(pi-p(brange)) + (~corr).*p(brange);
p(arange) = p(arange) + corr.*pi;
if maxabspcross == 0
    p(arange) = p(arange) - pi*floor(p(arange)/pi);
else
    p(arange) = p(arange) - 2*pi*floor(p(arange)/pi/2);
end
