%Generates a library of p functions (see Eq. 2), which are stored as column
%vectors of function values at rho values. Length is in units of nm. 

%Experimental parameters. 
NA = 1.45;
M = 150;
lambda = 580;
k = 2*pi/lambda;
n = 1.5;

%Variables of integration. 
etapmax = NA/M;
detap = etapmax/1000;
etap = 0:detap:etapmax;
eta = asin(M*sin(etap)/n);

%rho values. 
drhop = .1;
rhopmax = 2000;
rhop = (0:drhop:rhopmax)';

%See Eq. 7. 
kz = n*k*cos(eta);
kr = n*k*sin(eta);
Epperp = kr;
Eppara = kz;
Espara = -n*k*ones(size(Epperp));

%Variables of convenience. 
sqrtcoseta2 = sqrt(cos(eta));
etap2 = etap.^2;
etap3 = etap.^3;
etap4 = etap.^4;

%Initialize variables for ABC (Eq. 3) functions of rho, and their
%derivatives. 
IA = zeros(size(rhop));
IB = IA;
IC = IA;
DIA = IA;
DIB = IA;
DIC = IA;
D2IA = IA;
D2IB = IA;
D2IC = IA;
D3IA = IA;
D3IB = IA;
D3IC = IA;
blocksize = 10000; %The integrals wrt etap are too large, hence they are evaluated in blocks of rho values. 
for i = 1:ceil(length(rhop)/blocksize)
    tic
    if i == ceil(length(rhop)/blocksize)
        block = ((i-1)*blocksize+1):length(rhop);
    else
        block = (i-1)*blocksize+(1:blocksize);
    end
    
    %Evaluate Bessel functions in ABC integrals (Eq. 3) using recurrence.
    %rhop is a column vector, etap is a row vector, so x is an array. 
    x = k*rhop(block)*etap*M;
    J0 = besselj(0,x);
    J1 = besselj(1,x);
    J2 = 2*J1./x-J0;
    J2(isnan(J2)) = 0;
    J3 = 4*J2./x-J1;
    J3(isnan(J3)) = 0;
    J4 = 6*J3./x-J2;
    J4(isnan(J4)) = 0;
    J5 = 8*J4./x-J3;
    J5(isnan(J5)) = 0;

    %Evaluate integrals using Simpson's rule. integ is the integrand, which
    %is an array where rho varies along the columns, etap varies along the
    %rows. The integrals are wrt etap, so sum along the rows, to yield
    %column vectors. 
    integ = bsxfun(@times,etap.*(Eppara-Espara)./sqrtcoseta2,J0);
    IA(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = bsxfun(@times,etap.*(Eppara+Espara)./sqrtcoseta2,J2);
    IB(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = bsxfun(@times,etap.*Epperp./sqrtcoseta2,J1);
    IC(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*k*bsxfun(@times,etap2.*(Eppara-Espara)./sqrtcoseta2,-J1);
    DIA(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*k*bsxfun(@times,etap2.*(Eppara+Espara)./sqrtcoseta2,.5*(J1-J3));
    DIB(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*k*bsxfun(@times,etap2.*Epperp./sqrtcoseta2,.5*(J0-J2));
    DIC(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*k*k*bsxfun(@times,etap3.*(Eppara-Espara)./sqrtcoseta2,.5*(-J0+J2));
    D2IA(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*k*k*bsxfun(@times,etap3.*(Eppara+Espara)./sqrtcoseta2,.25*(J0-2*J2+J4));
    D2IB(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*k*k*bsxfun(@times,etap3.*Epperp./sqrtcoseta2,.25*(-J1+J3));
    D2IC(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*M*k*k*k*bsxfun(@times,etap4.*(Eppara-Espara)./sqrtcoseta2,.25*(J1-J3));
    D3IA(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*M*k*k*k*bsxfun(@times,etap4.*(Eppara+Espara)./sqrtcoseta2,.125*(-4*J1+3*J3-J5));
    D3IB(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;
    integ = M*M*M*k*k*k*bsxfun(@times,etap4.*Epperp./sqrtcoseta2,.125*(-3*J0+4*J2-J4));
    D3IC(block) = (integ(:,1) + 4*sum(integ(:,2:2:(end-1)),2) + 2*sum(integ(:,3:2:(end-2)),2) + integ(:,end))*detap/3;

    i
    toc
end

%Evaluate I constants (Eq. 4). 
integ = etap./cos(eta).*(abs(Eppara-Espara).^2+abs(Eppara+Espara).^2);
Ipara = pi/2*(integ(1) + 4*sum(integ(2:2:(end-1))) + 2*sum(integ(3:2:(end-2))) + integ(end))*detap/3/k/k/M/M;
integ = etap./cos(eta).*abs(Epperp).^2;
Iperp = 2*pi*(integ(1) + 4*sum(integ(2:2:(end-1))) + 2*sum(integ(3:2:(end-2))) + integ(end))*detap/3/k/k/M/M;

%Evaluate p functions and their derivatives. 
ppara = .25*(abs(IA).^2+abs(IB).^2);
pperp = abs(IC).^2;
pcross = imag((IB-IA).*conj(IC));
delppara = -.5*real(IA.*conj(IB));
Dppara = .5*(real(IA).*real(DIA)+imag(IA).*imag(DIA)+real(IB).*real(DIB)+imag(IB).*imag(DIB));
Dpperp = 2*(real(IC).*real(DIC)+imag(IC).*imag(DIC));
Dpcross = imag((IB-IA).*conj(DIC)+(DIB-DIA).*conj(IC));
Ddelppara = -.5*real(IA.*conj(DIB)+DIA.*conj(IB));
D2ppara = .5*(real(IA).*real(D2IA)+real(DIA).^2+imag(IA).*imag(D2IA)+imag(DIA).^2+real(IB).*real(D2IB)+real(DIB).^2+imag(IB).*imag(D2IB)+imag(DIB).^2);
D2pperp = 2*(real(IC).*real(D2IC)+real(DIC).^2+imag(IC).*imag(D2IC)+imag(DIC).^2);
D2pcross = imag((IB-IA).*conj(D2IC)+2*(DIB-DIA).*conj(DIC)+(D2IB-D2IA).*conj(IC));
D2delppara = -.5*real(IA.*conj(D2IB)+2*DIA.*conj(DIB)+D2IA.*conj(IB));
D3ppara = .5*(real(IA).*real(D3IA)+3*real(DIA).*real(D2IA)+imag(IA).*imag(D3IA)+3*imag(DIA).*imag(D2IA)+real(IB).*real(D3IB)+3*real(DIB).*real(D2IB)+imag(IB).*imag(D3IB)+3*imag(DIB).*imag(D2IB));
D3pperp = 2*(real(IC).*real(D3IC)+3*real(DIC).*real(D2IC)+imag(IC).*imag(D3IC)+3*imag(DIC).*imag(D2IC));
D3pcross = imag((IB-IA).*conj(D3IC)+3*(DIB-DIA).*conj(D2IC)+3*(D2IB-D2IA).*conj(DIC)+(D3IB-D3IA).*conj(IC));
D3delppara = -.5*real(IA.*conj(D3IB)+3*DIA.*conj(D2IB)+3*D2IA.*conj(DIB)+D3IA.*conj(IB));